/*-
 * Copyright (c) 2001
 * Tatsuya Kudoh(CDR/TK),ROYALPANDA.    All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _KS_KEYCODE_H_
#define _KS_KEYCODE_H_

#define KEYCODE_UP		0x15	/* ^U */
#define KEYCODE_DOWN		0x04	/* ^D */
#define KEYCODE_LEFT		0x0c	/* ^L */
#define KEYCODE_RIGHT		0x12	/* ^R */

#define KEYCODE_PRIOR		0x10	/* ^P */
#define KEYCODE_NEXT		0x0e	/* ^N */
#define KEYCODE_HOME		0x13	/* ^S */
#define KEYCODE_END		0x05	/* ^E */

#define KEYCODE_HIRAGANA	0x07	/* ^G */
#define KEYCODE_KATAKANA	0x0b	/* ^K */
#define KEYCODE_ASCII		0x01	/* ^A */
#define KEYCODE_OPTION		0x0f	/* ^O */
#define KEYCODE_CONVERSION	0x03	/* ^C */
#define KEYCODE_SELECT		0x0d	/* ^M */

#define KEYCODE_BACKSPACE	0x08	/* ^H */
#define KEYCODE_TAB		0x09	/* ^I */
#define KEYCODE_RETURN		0x0a	/* ^J */
#define KEYCODE_SPACE		0x20	/*    */

#define KEYCODE_TOGGLECONV	0x14	/* ^T */
#define KEYCODE_NOCONV		0x19	/* ^Y */
#define KEYCODE_DELETEDIC	0x18	/* ^X */

#define KEYCODE_REDRAW		0x17	/* ^W */

#define KEYCODE_FILEEND		0x1a	/* ^Z */


#define KEYSTR_UP		"\x15"	/* ^U */
#define KEYSTR_DOWN		"\x04"	/* ^D */
#define KEYSTR_LEFT		"\x0c"	/* ^L */
#define KEYSTR_RIGHT		"\x12"	/* ^R */

#define KEYSTR_PRIOR		"\x10"	/* ^P */
#define KEYSTR_NEXT		"\x0e"	/* ^N */
#define KEYSTR_HOME		"\x13"	/* ^S */
#define KEYSTR_END		"\x05"	/* ^E */

#define KEYSTR_HIRAGANA		"\x07"	/* ^G */
#define KEYSTR_KATAKANA		"\x0b"	/* ^K */
#define KEYSTR_ASCII		"\x01"	/* ^A */
#define KEYSTR_OPTION		"\x0f"	/* ^O */
#define KEYSTR_CONVERSION	"\x03"	/* ^C */
#define KEYSTR_SELECT		"\x0d"	/* ^M */

#define KEYSTR_BACKSPACE	"\x08"	/* ^H */
#define KEYSTR_TAB		"\x09"	/* ^I */
#define KEYSTR_RETURN		"\x0a"	/* ^J */
#define KEYSTR_SPACE		"\x20"	/*    */

#define KEYSTR_TOGGLECONV	"\x14"	/* ^T */
#define KEYSTR_NOCONV		"\x19"	/* ^Y */
#define KEYSTR_DELETEDIC	"\x18"	/* ^X */

#define KEYSTR_FILEEND		"\x1a"	/* ^Z */

#define KEYSTR_REDRAW		"\x17"	/* ^W */

#endif
